/*

Copyright (C) 2004 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef __libnd_phdl_exp_h
#define __libnd_phdl_exp_h

#include <libnd_phdl.h>
#include <libnd_phdl_header.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum {
  LND_PHDL_ADD,
  LND_PHDL_SUB,
  LND_PHDL_MUL,
  LND_PHDL_DIV,
  LND_PHDL_MOD,
  LND_PHDL_NON
} LND_PHDL_NumOp;

typedef enum {
  LND_PHDL_EQU, /* =  */
  LND_PHDL_NEQ, /* != */
  LND_PHDL_LET, /* <  */
  LND_PHDL_GET, /* >  */
  LND_PHDL_LEQ, /* <= */
  LND_PHDL_GEQ, /* >= */
  LND_PHDL_NOB  /* No op, in case of one operand only */
} LND_PHDL_BoolOp;

typedef enum {
  LND_PHDL_LITERAL,
  LND_PHDL_FIELD,
  LND_PHDL_EXP
} LND_PHDL_ParamType;

typedef struct lnd_phdl_numexp LND_PHDL_NumExp;
typedef struct lnd_phdl_boolexp LND_PHDL_BoolExp;

typedef struct lnd_phdl_numparam
{
  LND_PHDL_ParamType    type;
  int                   literal;
  char                 *field_name;
  LND_PHDL_NumExp      *exp;
} LND_PHDL_NumParam;

struct lnd_phdl_numexp
{
  LND_PHDL_NumParam    *param1;
  LND_PHDL_NumParam    *param2;
  LND_PHDL_NumOp        op;
};

struct lnd_phdl_boolexp
{
  LND_PHDL_NumParam    *param1;
  LND_PHDL_NumParam    *param2;
  LND_PHDL_BoolOp       op;
};


LND_PHDL_NumExp    *libnd_phdl_numexp_new(LND_PHDL_NumParam *param1,
					  LND_PHDL_NumParam *param2,
					  LND_PHDL_NumOp op);
void                libnd_phdl_numexp_free(LND_PHDL_NumExp *exp);

gboolean            libnd_phdl_numexp_eval(const LND_PHDL_NumExp *exp,
					   LND_PHDL_Header *header,
					   guint64 *result);

const char         *libnd_phdl_numexp_to_string(const LND_PHDL_NumExp *exp);


LND_PHDL_BoolExp   *libnd_phdl_boolexp_new(LND_PHDL_NumParam *param1,
					   LND_PHDL_NumParam *param2,
					   LND_PHDL_BoolOp op);
void                libnd_phdl_boolexp_free(LND_PHDL_NumExp *exp);

gboolean            libnd_phdl_boolexp_eval(const LND_PHDL_BoolExp *exp,
					    LND_PHDL_Header *header,
					    gboolean *result);

const char         *libnd_phdl_boolexp_to_string(const LND_PHDL_BoolExp *exp);


LND_PHDL_NumParam  *libnd_phdl_numparam_lit_new(int value);
LND_PHDL_NumParam  *libnd_phdl_numparam_exp_new(LND_PHDL_NumExp *exp);
LND_PHDL_NumParam  *libnd_phdl_numparam_field_new(const char *field_name);
void                libnd_phdl_numparam_free(LND_PHDL_NumParam *param);
const char         *libnd_phdl_numparam_to_string(const LND_PHDL_NumParam *param);
gboolean            libnd_phdl_numparam_eval(LND_PHDL_NumParam *param,
					     LND_PHDL_Header *header,
					     guint64 *result);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __libnd_phdl_exp_h */
