/*

Copyright (C) 2004 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
%option noyywrap

%{
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <libnd.h>
#include "libnd_phdl.h"
#include "libnd_phdl_parse.h"

int phdlerror(char *fmt, ...);
#define yyerror phdlerror

extern int parse_lineno;

%}

%%
[ \t]           { }
def		{ return DEF; }
proto		{ return PROTO; }
hex		{ return HEX; }
int		{ return INT; }
unsigned        { return UNSIGNED; }
str             { return STR; }
enum            { return ENUM; }
other           { return OTHER; }
unit            { return UNIT; }
scale           { return SCALE; }
sep		{ return SEP; }
chain		{ return CHAIN; }
until           { return UNTIL; }
items           { return ITEMS; }
seen            { return SEEN; }
length          { return LENGTH; }
union		{ return UNION; }
block		{ return BLOCK; }
if		{ return IF; }
link		{ return LINK; }
net		{ return NET; }
trans		{ return TRANS; }
app		{ return APP; }



==		{ return EQU; }
!=		{ return NEQ; }
\<		{ return LET; }
>		{ return GET; }
\<=		{ return LEQ; }
>=		{ return GEQ; }
TRUE		{ return TRU; }
FALSE		{ return FLS; }

0x[0-9a-fA-F]+  { yylval.number = strtol(yytext, NULL, 16);
                  /* D(("Number %i lexed.\n", (int) yylval.number)); */
                  return HEXNUMBER; }
[0-9]+		{ yylval.number = atoi(yytext);
                  /* D(("Number %i lexed.\n", (int) yylval.number)); */
                  return NUMBER; }
\"[^\"]+\"      { yylval.string = strdup(yytext + 1);
                  yylval.string[strlen(yylval.string) - 1] = '\0';
                  /* D(("String '%s' lexed.\n", yylval.string)); */
                  return STRING; }
[a-zA-Z.][0-9a-zA-Z.]+ { yylval.string = strdup(yytext);
                         yylval.string[strlen(yylval.string)] = '\0';
                         /* D(("Fieldname '%s' lexed.\n", yylval.string)); */
                         return FIELD; }
"#".*\n         { parse_lineno++; }
"//".*\n        { parse_lineno++; }
\n              { parse_lineno++; }
\(              { return '('; }
\)              { return ')'; }
:               { return ':'; }
\{              { return '{'; }
\}              { return '}'; }
!               { return '!'; }
;               { return ';'; }
=               { return '='; }
\+              { return '+'; }
-               { return '-'; }
\*              { return '*'; }
\/              { return '/'; }
%               { return '%'; }

.               { D(("Syntax error: '%s'\n", yytext)); }

%%
